eval 'exec perl -x $0 ${1+"$@"}' # -*-perl-*-
  if 0;
#!perl -w
#
# ======================================================================
# This file is Copyright 1998,1999 by the Purdue Research Foundation and
# may only be used under license.  For terms of the license, see the
# file named COPYRIGHT included with this software release.
# AAFID is a trademark of the Purdue Research Foundation.
# All rights reserved.
# ======================================================================
#
# AAFID::PlainTransceiver package.
#
# AAFID project, COAST Laboratory, CERIAS, 1998-1999.
# 
# Diego Zamboni, Jan 27, 1998.
#
# NOTE: This file is in Perl's POD format. For more information, see the 
#       manual page for perlpod(1).
#

package AAFID::PlainTransceiver;

# The following keeps up with the RCS version number. The self-assignment
# keeps the -w switch from complaining (because $VERSION may not be used
# here, but it is used in our base class).
$VERSION = do { my @r = (q$Revision: 1.9 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; $VERSION = $VERSION;

%PARAMETERS = (
	       Description => "Simple Transceiver",
	      );
  
use vars qw(
	    @ISA
	    $VERSION
	    %PARAMETERS
	   );
use strict;
use AAFID::Config;
use AAFID::Entity;
use AAFID::ControllerEntity;
use AAFID::Constants;
use AAFID::Comm;
use Carp;

@ISA=qw(AAFID::ControllerEntity
	AAFID::Log);

=pod

Simple transceiver. For now, it is an empty subclass of C<ControllerEntity>,
and thus implements the same functionality: it is able to start agents,
and then listens to whatever they say and sends it up.

=cut

_EndOfEntity;

#
# $Id: PlainTransceiver.pm,v 1.9 1999/09/03 17:08:54 zamboni Exp $
# $Log: PlainTransceiver.pm,v $
# Revision 1.9  1999/09/03 17:08:54  zamboni
# Changed the start line to something that is path-independent, and
# updated the copyright notice.
#
# Revision 1.8  1999/03/29 22:33:27  zamboni
# Merged branch a05-new-comm-module, which updates it to make use of the new event-based communication mechanism.
#
# Revision 1.7.4.1  1999/03/29 16:41:50  zamboni
# - Moved the Log tag to the end of the file.
#
# Revision 1.7  1998/06/29 20:11:23  zamboni
# Added copyright message
#
# Revision 1.6  1998/06/26 21:31:11  zamboni
# - Added "use AAFID::Config"
#
# Revision 1.5  1998/03/13 16:59:51  zamboni
# - Added $VERSION and %PARAMETERS
#
# Revision 1.4  1998/03/06 07:12:03  zamboni
# - Added "use strict"
#
# Revision 1.3  1998/03/05 06:30:05  zamboni
# - Removed a duplicate "use AAFID::Comm"
#
# Revision 1.2  1998/03/05 04:29:04  zamboni
# Renamed the class to AAFID::PlainTransceiver, to leave option for other
# kinds of transceivers.
#
# Revision 1.1  1998/03/05 04:28:15  zamboni
# Initial revision
#
# Revision 1.4  1998/03/04 22:12:34  zamboni
# Complete rewrite of the class. Now, in its simplest form, it is an
# empty subclass of AAFID::ControllerEntity.
#
# Revision 1.3  1998/01/27 19:21:01  zamboni
# First running version. Receives messages correctly.
#
# Revision 1.2  1998/01/27 19:16:21  zamboni
# Corrected a minor error (hadn't done the "use Carp")
#
# Revision 1.1  1998/01/27 19:15:22  zamboni
# Initial revision
#
