eval 'exec perl -x $0 ${1+"$@"}' # -*-perl-*-
  if 0;
#!perl -w
#
# ======================================================================
# This file is Copyright 1998,1999 by the Purdue Research Foundation and
# may only be used under license.  For terms of the license, see the
# file named COPYRIGHT included with this software release.
# AAFID is a trademark of the Purdue Research Foundation.
# All rights reserved.
# ======================================================================
#
# AAFID::Constants package
#
# AAFID project, COAST Laboratory, CERIAS, 1998-1999.
# 
# Diego Zamboni, Jan 27, 1998.
#
# $Id: Constants.pm,v 1.8 1999/09/03 17:08:53 zamboni Exp $
# $Log: Constants.pm,v $
# Revision 1.8  1999/09/03 17:08:53  zamboni
# Changed the start line to something that is path-independent, and
# updated the copyright notice.
#
# Revision 1.7  1998/06/29 20:11:23  zamboni
# Added copyright message
#
# Revision 1.6  1998/05/02 19:50:48  zamboni
# Corrected a syntax problem in the POD part.
#
# Revision 1.5  1998/03/13 17:01:35  zamboni
# - Added $VERSION declaration.
#
# Revision 1.4  1998/03/06 07:12:31  zamboni
# - Added "use strict"
#
# Revision 1.3  1998/03/05 05:13:43  zamboni
# Added the missing 1; at the end.
#
# Revision 1.2  1998/03/05 05:13:26  zamboni
# Changed the name of TRANSCEIVER_PORT to CONTROLLERENTITY_PORT. Yes, it's
# long. You don't want to use it too much.
#
# Revision 1.1  1998/01/27 19:06:22  zamboni
# Initial revision
#
#
# NOTE: This file is in Perl's POD format. For more information, see the 
#       manual page for perlpod(1).
#

package AAFID::Constants;

# The following keeps up with the RCS version number. The self-assignment
# keeps the -w switch from complaining (because $VERSION may not be used
# here, but it is used in our base class).
$VERSION = do { my @r = (q$Revision: 1.8 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; $VERSION = $VERSION;

use vars qw(
	    $CONTROLLERENTITY_PORT
	    @ISA
	    @EXPORT
	   );
use strict;
use Exporter();

@ISA=qw(Exporter);

@EXPORT=qw(
	   $CONTROLLERENTITY_PORT
	  );

=pod

This module defines a number of constants that are used throughout the
AAFID system as parameters.

=over 4

=item CONTROLLERENTITY_PORT

The port in which a controller entity listens to for new connections from
entities in other hosts.

=cut

    $CONTROLLERENTITY_PORT=44777;

=back

=cut

1;
